import yaml
import os

__all__ = ["Settings"]


class Settings(object):
    """Class for parsing and representate settings
    """

    class YAMLParseException(Exception):
        pass

    def update_from_yaml(self, yaml_filename: str):
        """
        :param yaml_filename
        :type yaml_filename: str
        """
        yaml_str = open(yaml_filename).read()
        yaml_dict = yaml.load(yaml_str)
        if not isinstance(yaml_dict, dict):
            raise Settings.YAMLParseException("YAML must be dict")
        self.__dict__.update(yaml_dict)
        if "$import" in yaml_dict:
            for import_file in yaml_dict["$import"]:
                try:
                    self.update_from_yaml(os.path.join(os.path.dirname(yaml_filename), import_file))
                except FileNotFoundError:
                    print("%s, imported from %s not found" % (import_file, yaml_filename))

    @staticmethod
    def from_yaml(yaml_filename: str):
        settings = Settings()
        settings.update_from_yaml(yaml_filename)
        return settings



import argparse
import asyncio
from sys import modules
from importlib import import_module, util
from puer.settings import Settings
from .scripts import scripts


__all__ = ['Manager']


class Manager(object):
    def __init__(self, args: argparse.Namespace, loop=None):
        self.args = args
        self.config_name = "%s.yaml" % self.args.config_name
        self.script_name = self.args.script_name
        self.settings = Settings.from_yaml(self.config_name)
        self.scripts = scripts
        self.loop = loop or asyncio.get_event_loop()
        self._load_apps(self.settings.apps)
        script = self.scripts[self.script_name](self)
        script.main()

    @staticmethod
    def parse_args():
        arg_parser = argparse.ArgumentParser(description='Manage script for Puer framework')
        arg_parser.add_argument('config_name', metavar='config_name', help='configuration file name '
                                                                 '(extension must be ".yaml")')
        arg_parser.add_argument('script_name', metavar='script_name', help='configuration name')
        return arg_parser.parse_args()

    def _load_apps(self, apps: list):
        for app in apps:
            self._load_scripts_from_app(app)
            self._load_urls_from_app(app)

    def _load_scripts_from_app(self, app):
        package_name = "%s.scripts" % app
        if util.find_spec(package_name) is not None:
            import_module(package_name)
            self.scripts.update(modules[package_name].scripts)

    def _load_urls_from_app(self, app):
        package_name = "%s.urls" % app
        if util.find_spec(package_name) is not None:
            import_module(package_name, app)



__all__ = ["route", "routes"]


routes = []


def route(r, func, name=None):
    """
    
    :param t: (str) http method
    :param r: (str) route
    :param func: (class|async function) handler
    :param name: route name
    :return: 
    """
    routes.append(("*", r, func, name))
